'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.MenuState = exports.Edge = exports.Node = exports.Pin = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _immutable = require('immutable');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaultPin = {
    name: '',
    connected: false,
    data: new _immutable.Map()
};

var Pin = exports.Pin = (0, _immutable.Record)(defaultPin);

var defaultNode = {
    id: 0,
    title: '',
    data: new _immutable.Map(),

    x: 0,
    y: 0,
    width: 0,
    height: 0,

    minPin: 1 / 0,
    pinHeight: 0,

    inputs: new _immutable.List(),
    outputs: new _immutable.List()
};

var Node = exports.Node = (0, _immutable.Record)(defaultNode);

var defaultEdge = {
    from: 0,
    output: 0,
    to: 0,
    input: 0,
    color: '#fff'
};

var Edge = exports.Edge = (0, _immutable.Record)(defaultEdge);

var defaultEditorState = {
    nodes: new _immutable.Map(),
    edges: new _immutable.List(),
    selection: new _immutable.List()
};

var EditorState = (0, _immutable.Record)(defaultEditorState);

var defaultMouseState = {
    down: false,
    x: 0,
    y: 0,
    startX: 0,
    startY: 0,

    node: null,
    pin: null
};

var MouseState = function (_Record) {
    _inherits(MouseState, _Record);

    function MouseState() {
        _classCallCheck(this, MouseState);

        return _possibleConstructorReturn(this, (MouseState.__proto__ || Object.getPrototypeOf(MouseState)).apply(this, arguments));
    }

    _createClass(MouseState, [{
        key: 'draggingEdge',
        get: function get() {
            return this.node !== null && this.pin !== null;
        }
    }, {
        key: 'rect',
        get: function get() {
            return {
                minX: Math.min(this.startX, this.x),
                minY: Math.min(this.startY, this.y),
                maxX: Math.max(this.startX, this.x),
                maxY: Math.max(this.startY, this.y)
            };
        }
    }]);

    return MouseState;
}((0, _immutable.Record)(defaultMouseState));

var defaultMenuState = {
    open: false,
    x: 0,
    y: 0
};

var MenuState = exports.MenuState = (0, _immutable.Record)(defaultMenuState);

var defaultGraphState = {
    editorState: new EditorState(),

    undoStack: new _immutable.Stack(),
    redoStack: new _immutable.Stack(),

    mouseState: new MouseState(),
    menuState: new MenuState()
};

var GraphState = function (_Record2) {
    _inherits(GraphState, _Record2);

    function GraphState() {
        _classCallCheck(this, GraphState);

        return _possibleConstructorReturn(this, (GraphState.__proto__ || Object.getPrototypeOf(GraphState)).apply(this, arguments));
    }

    _createClass(GraphState, [{
        key: 'save',
        value: function save() {
            return this.editorState.toJS();
        }
    }, {
        key: 'addNode',
        value: function addNode(node) {
            var _menuState = this.menuState,
                x = _menuState.x,
                y = _menuState.y;


            return this.__pushState(this.editorState.update('nodes', function (nodes) {
                var data = new Node(_extends({}, node, {
                    id: nodes.size,
                    x: x,
                    y: y
                }));

                return nodes.set(data.id, data);
            }));
        }
    }, {
        key: '_measureNode',
        value: function _measureNode(id, width, height) {
            return this.updateIn(['editorState', 'nodes', id], function (node) {
                return node.set('width', width).set('height', height);
            });
        }
    }, {
        key: '_measurePin',
        value: function _measurePin(id, y, height) {
            return this.updateIn(['editorState', 'nodes', id], function (node) {
                return node.update('minPin', function (v) {
                    return Math.min(v, y - node.y);
                }).set('pinHeight', height);
            });
        }
    }, {
        key: 'moveNode',
        value: function moveNode(id, x, y) {
            var _this3 = this;

            var pushUndo = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

            var nextState = this.isSelected(id) ? this.editorState.update('nodes', function (nodes) {
                var node = nodes.get(id);
                var deltaX = x - node.x;
                var deltaY = y - node.y;

                return nodes.map(function (n) {
                    if (_this3.isSelected(n.id)) {
                        return n.set('x', n.x + deltaX).set('y', n.y + deltaY);
                    }

                    return n;
                });
            }) : this.editorState.set('selection', new _immutable.List([id])).updateIn(['nodes', id], function (node) {
                return node.set('x', x).set('y', y);
            });

            if (pushUndo) {
                return this.__pushState(nextState);
            }

            return this.set('editorState', nextState);
        }
    }, {
        key: 'mapNodes',
        value: function mapNodes(fn) {
            return this.updateIn(['editorState', 'nodes'], function (nodes) {
                return nodes.map(fn);
            });
        }
    }, {
        key: '_startConnection',
        value: function _startConnection(node, pin) {
            return this.update('mouseState', function (mouse) {
                return mouse.set('node', node.id).set('pin', node.outputs.findKey(function (_ref) {
                    var name = _ref.name;
                    return name === pin.name;
                }));
            });
        }
    }, {
        key: '_endConnection',
        value: function _endConnection(node, pin) {
            var from = this.mouseState.node;
            var output = this.mouseState.pin;

            var to = node.id;
            var input = node.inputs.findKey(function (_ref2) {
                var name = _ref2.name;
                return name === pin.name;
            });

            return this._endMouse().addLink(from, output, to, input);
        }
    }, {
        key: 'addLink',
        value: function addLink(from, output, to, input) {
            return this.__pushState(this.editorState.update('nodes', function (nodes) {
                return nodes.updateIn([from, 'outputs', output], function (pin) {
                    return pin.set('connected', true);
                }).updateIn([to, 'inputs', input], function (pin) {
                    return pin.set('connected', true);
                });
            }).update('edges', function (edges) {
                return edges.filter(function (edge) {
                    return edge.to !== to || edge.input !== input;
                }).push(new Edge({
                    id: edges.size,
                    from: from,
                    to: to,
                    output: output,
                    input: input
                }));
            }));
        }
    }, {
        key: 'openMenu',
        value: function openMenu(x, y) {
            return this.update('menuState', function (menu) {
                return menu.set('open', true).set('x', x).set('y', y);
            });
        }
    }, {
        key: 'closeMenu',
        value: function closeMenu() {
            return this.setIn(['menuState', 'open'], false);
        }
    }, {
        key: 'selectNode',
        value: function selectNode(id) {
            var pushNode = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

            return this.__pushState(this.editorState.update('selection', function (selection) {
                if (pushNode) {
                    return selection.push(id);
                }

                return new _immutable.List([id]);
            }));
        }
    }, {
        key: 'isSelected',
        value: function isSelected(node) {
            return this.editorState.selection.find(function (id) {
                return id === node;
            }) !== undefined;
        }
    }, {
        key: 'clearSelection',
        value: function clearSelection() {
            return this.editorState.selection.isEmpty() ? this : this.__pushState(this.editorState.update('selection', function (list) {
                return list.clear();
            }));
        }
    }, {
        key: '_startMouse',
        value: function _startMouse(x, y) {
            return this.update('mouseState', function (mouse) {
                return mouse.set('down', true).set('startX', x).set('startY', y);
            })._updateMouse(x, y);
        }
    }, {
        key: '_updateMouse',
        value: function _updateMouse(x, y) {
            var nextState = this.update('mouseState', function (mouse) {
                return mouse.set('x', x).set('y', y);
            });

            var _nextState$mouseState = nextState.mouseState,
                node = _nextState$mouseState.node,
                pin = _nextState$mouseState.pin;


            if (node == null && pin == null) {
                var _ret = function () {
                    var _nextState$mouseState2 = nextState.mouseState.rect,
                        minX = _nextState$mouseState2.minX,
                        minY = _nextState$mouseState2.minY,
                        maxX = _nextState$mouseState2.maxX,
                        maxY = _nextState$mouseState2.maxY;


                    return {
                        v: nextState.setIn(['editorState', 'selection'], nextState.editorState.nodes.filter(function (n) {
                            return n.x >= minX && n.x + n.width <= maxX && n.y >= minY && n.y + n.height <= maxY;
                        }).map(function (_ref3) {
                            var id = _ref3.id;
                            return id;
                        }))
                    };
                }();

                if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
            }

            return nextState;
        }
    }, {
        key: '_endMouse',
        value: function _endMouse() {
            var nextState = this;
            if (!this.mouseState.draggingEdge) {
                nextState = this.__pushState(this.editorState);
            }

            return nextState.update('mouseState', function (mouse) {
                return mouse.set('down', false).set('node', null).set('pin', null);
            });
        }
    }, {
        key: '__pushState',
        value: function __pushState(nextState) {
            var _this4 = this;

            return this.update('undoStack', function (stack) {
                return stack.push(_this4.editorState);
            }).set('editorState', nextState);
        }
    }, {
        key: 'undo',
        value: function undo() {
            if (this.undoStack.isEmpty()) {
                return this;
            }

            var prevState = this.undoStack.peek();
            var currState = this.editorState;

            return this.set('editorState', prevState).update('undoStack', function (stack) {
                return stack.pop();
            }).update('redoStack', function (stack) {
                return stack.push(currState);
            });
        }
    }, {
        key: 'redo',
        value: function redo() {
            if (this.redoStack.isEmpty()) {
                return this;
            }

            var currState = this.editorState;
            var nextState = this.redoStack.peek();

            return this.set('editorState', nextState).update('undoStack', function (stack) {
                return stack.push(currState);
            }).update('redoStack', function (stack) {
                return stack.pop();
            });
        }
    }, {
        key: 'selectedNodes',
        get: function get() {
            var _this5 = this;

            return this.editorState.selection.map(function (id) {
                return _this5.editorState.nodes.get(id);
            });
        }
    }], [{
        key: 'createEmpty',
        value: function createEmpty() {
            return new GraphState();
        }
    }, {
        key: 'fromGraph',
        value: function fromGraph(nodes, edges) {
            return new GraphState({
                editorState: new EditorState({
                    nodes: nodes, edges: edges
                })
            });
        }
    }, {
        key: 'restore',
        value: function restore(data) {
            return new GraphState({
                editorState: (0, _immutable.fromJS)(data, function (key, value) {
                    switch (key) {
                        case '':
                            return new EditorState(value.toObject());

                        case 'selection':
                            return value.toList();

                        case 'nodes':
                            return value.toMap().mapEntries(function (_ref4) {
                                var _ref5 = _slicedToArray(_ref4, 2),
                                    k = _ref5[0],
                                    v = _ref5[1];

                                return [Number(k), new Node(v.toObject())];
                            });

                        case 'edges':
                            return value.toList().map(function (edge) {
                                return new Edge(edge.toObject());
                            });

                        case 'inputs':
                        case 'outputs':
                            return value.toList().map(function (pin) {
                                return new Pin(pin);
                            });

                        case 'data':
                            return value.toMap();

                        default:
                            {
                                var isIndexed = _immutable.Iterable.isIndexed(value);
                                return isIndexed ? value.toList() : value.toMap();
                            }
                    }
                })
            });
        }
    }]);

    return GraphState;
}((0, _immutable.Record)(defaultGraphState));

exports.default = GraphState;